/*******************************************************************************
		Main Program:					Viewer.cpp

		Comment:							Basic LString Drawer

		Author:								Yu-Chi Lai, yu-chil@cae.wisc.edu

		Due Date:			

		Course:								CS777, Spring 2003
*******************************************************************************/

#ifndef LTURTLE_H
#define LTURTLE_H
// Include necessary file
#include "Lstring.h"
#include "../global/globals.h"
#include "../math/vect3.h"

#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <math.h>
#include <vector>
#include <stack>
#include <fstream>


typedef std::vector<vect3<float> > p_rec;

// Settings stack used for solving [] references 
typedef struct s_rec {
    vect3<float>	pos;            // position in 3space of turtle origin 
    vect3<float>	fow;            // forward direction 
    vect3<float>	lef;            // left direction 
    vect3<float>	upp;            // up direction 
    vect3<float>	last;           // last position used for connecting cylinders 
    vect3<float>	last_v[9];      // last vertices of object used for connecting cylinders */
    float32				dis;						// value of F distance 
    float32				ang;						// value of basic angle 
    float32				thick;					// value of thickness 
		float32       thickness;
		float32       oldThickness;		// The previous draw thickness
		bool          thickInit;
    float32				dis2;						// value of Z distance 
    float32				tr;							// trope value ?????????????????
    uint16				col;						// current color 
    uint16				last_col;				// color of last object 
} s_rec;

struct Point2D{
 double  x, y;
};

class LTurtle
{
	public:
		// Constructor and destructor
		LTurtle(LString &inLString);
		~LTurtle();
			
		// Draw the LString
		bool  Draw();

		// Set up the random seed
		void  SetRandSeed(int i);

		// Create default string
		void	createString();
		// Set the default angle
		void  setAngle(double angle);
		// Set the length Ratio
		void  setLengthRatio(double lr);
		// Set the length deviation
		void	setLengthDeviation(double ld);
		// Set the angle ratio
		void  setAngleRatio(double ar);
		// set the angle deviation
		void	setAngleDeviation(double ad);
		// Set up the LString from input file
		void	setLString(const char *filename);
		// Set up the LString From select type
		void  setLString(int index);
		// Set up the default tree height
		void  setTreeHeight(double h);
		// Set up the default leaf length
		void  setLeafLength(double l);
		// Set up the total tree numbers
		void  setNumber(int i);
		// Set up the number of mutation
		void  setMutation(int m);
		// Set up the recursion time
		void  setRecursion(int r);
		// Set up the width of tree
		void  setWidth(double width);
		// Set up the length of tree
		void  setLength(double length);
		// Set up the position of x, y, z
		void  setPositionX(double x);
		void  setPositionY(double y);
		void  setPositionZ(double z);
		void  printDefault();

	private:
		// Check whether the String and Graphic has been set up
		bool  CheckSetup();

		// Plot the LString
		void  PlotLstring(double *centre);
		// Plot the branch
		void  Define_block(vect3<float> p1, vect3<float> p2, vect3<float> up, float32 thick, float32 oldThickness,  float32 min_thick, uint16 c); //draws a cuboid
		// Plot the polygon
		void  Define_triangles(p_rec& ver, uint16 c); //draws a triangle strip
		// Get rotation
		void  Set_rot(float32 a, vect3<float> n);
		// Generate random position for forest
		void	generateRandomPosition();
	
	private:
		LString&		the_LString;					// Operating string variable
		std::vector<std::string> pattern; // The default tree and other pattern
			
		int           RandSeed;						// The seed of the random variable
		float32				rand_amount;				//
		float32				trope_amount;				//
		vect3<float>  trope, C1, C2, C3;	//

		vect3<float>  sky;								// The sky direction
		//***************************************************
		// The following is the mutation setting parameter
		//***************************************************
		double        DAngle;			// override the default angle
		double				DLRatio;		// Length variation ratio
		double        DLDeviation;// Variation deviation
		double        DARatio;		// Angle variation ratio
		double        DADeviation;// variation deviation
		double        DPosition[3];// plant position
		double        DTreeHeight;// the length of each segment
		double        DLeafLength;// the length of leaf
		int		        DMutation;	// The number of mutation
		int						DRecursion;	// The number of recursion
		int						DNum;				// The number of tree
		double        DWidth;			// The width of distributed field
		double        DLength;		// The length of the distributed field
		bool					initFlag;		// Whent some string have been initial
		std::vector<Point2D> TreePosition;// The Tree Position
};


#endif
