#include "Defines.h"
#include "function.h"

static int cube_gons[24] = {
    1,4,3,2,
    8,5,6,7,
    3,7,6,2,
    1,5,8,4,
    3,4,8,7,
    2,6,5,1
};
//Vertices used for each side of the cube 
static int side[10][4] = {{ 2, 3, 13, 12}, 
													{ 3, 4, 14, 13}, 
													{ 4, 5, 15, 14},
													{ 5, 6, 16, 15},
													{ 6, 7, 17, 16},
													{ 7, 8, 18, 17},
													{ 8, 9, 19, 18},
													{ 9,10, 20, 19},
													{10,11, 21, 20},
													{11, 2, 12, 21}};

//Vertices used for each side of the cube 
static int bottom[10][3] = {	{ 0, 3, 2}, 
																{ 0, 4, 3}, 
																{ 0, 5, 4},
																{ 0, 6, 5},
																{ 0, 7, 6},
																{ 0, 8, 7},
																{ 0, 9, 8},
																{ 0,10, 9},
																{ 0,11,10},
																{ 0, 2,11}};

//Vertices used for each side of the cube 
static int top[10][3] = {	{ 1, 12, 13}, 
													{ 1, 13, 14}, 
													{ 1, 14, 15},
													{ 1, 15, 16},
													{ 1, 16, 17},
													{ 1, 17, 18},
													{ 1, 18, 19},
													{ 1, 19, 20},
													{ 1, 20, 21},
													{ 1, 21, 12}};

extern std::vector<MString> colorTable;

//*****************************************************************************
// Removes a line from "text", and returns the removed line
// as a string, minus leading whitespace.
//*****************************************************************************
std::string snip_line(std::string &text)
{
	// The end of the string
  size_t marked_pos;
	// The temporary storing string
	std::string ss;

	// The text contain nothing
  if (text.length()==0)
    return "";

  // Find the end position of the line
  marked_pos=text.find_first_of(endline_markers);
	// text contain only one line
  if (marked_pos == std::string::npos) {
    ss = text;
    text = "";
  }
	// Copy this text and move the cursor of text to proper place
	else {                       //end of line marker found
		// Extract the text out
    ss         = text.substr(0, marked_pos);
		// Find the next element after the end of line
    marked_pos = text.find_first_not_of(endline_markers, marked_pos);
		// Erase the ss out from text
    text.erase(0, marked_pos);
  };
  
  //strip leading whitespace
  marked_pos =  ss.find_first_not_of(whitespace);
  ss.erase(0, marked_pos);
  
  return ss;
}


MStatus BlockDraw(char *name, std::vector<vect3<float> > &vertex, unsigned short int colorIndex)
{
	
	// Misc. primitive data
	int num_verts;								// Number of vertices of polygon
	int num_TopFaces;							// Number of faces on polygon
	int num_BottomFaces;					// Number of faces on polygon
	int num_SideFaces;						// Number of faces on polygon


	int num_edges;								// Number of edges on polygon
	int topBottomEdgePerFaces = 3;
	int sideEdgePerFaces			= 4;
	int num_face_connects;				// Number of elements in face connect array

	
	MStatus stat;
	MFnMesh					 fnPoly;			// create a poly function set 
	MFloatPointArray iarr;				// To put the vertices point
	MFloatPointArray pa;
	MIntArray				 faceCounts;
	MIntArray				 faceConnects;

	MObject					 newTransform;
	MDGModifier			 dgModifier;

	iarr.clear();
  // First, create the points:
	if(vertex.size() == 22){
		for(int i = 0; i < vertex.size(); i++){
			MFloatPoint pnt( vertex[i][0], vertex[i][1], vertex[i][2]);
			iarr.append(pnt);
		}
	}

	
	num_verts				 = 22;
	num_TopFaces		 = 10;
	num_BottomFaces  = 10;
	num_SideFaces    = 10;

	// Construct the point array
	pa.clear();
	int i, j;
	for (i = 0; i < num_verts; i++){
		pa.append(iarr[i]);
	}

	// If we are using polygon data then set up the face connect array
	// here. Otherwise, the create function will do it.
	num_face_connects = (num_TopFaces + num_BottomFaces) * topBottomEdgePerFaces +
										   num_SideFaces * sideEdgePerFaces;
	num_edges = num_face_connects / 2;

	// Append the edge per faces For tope
	for ( i = 0; i < num_TopFaces; i++ ){
		faceCounts.append(topBottomEdgePerFaces);
		for(j = 0; j < topBottomEdgePerFaces; j++){
			faceConnects.append(top[i][j]);
		}
	}

	// Append the edge per faces For Bottom
	for ( i = 0; i < num_BottomFaces; i++ ){
		faceCounts.append(topBottomEdgePerFaces);
		for(j = 0; j < topBottomEdgePerFaces; j++)
			faceConnects.append(bottom[i][j]);
	}
	// Append the edge per faces for side
	for ( i = 0; i < num_SideFaces; i++ ){
		faceCounts.append(sideEdgePerFaces);
		for(j = 0; j < sideEdgePerFaces; j++)
			faceConnects.append(side[i][j]);
	}

	// Call the poly creation method to create the polygon
	newTransform = fnPoly.create(num_verts, num_TopFaces + num_BottomFaces + num_SideFaces,
															 pa, faceCounts, faceConnects, MObject::kNullObj, &stat );

	checkErr(stat, "Could not create MFnMesh");

	// Primitive is created so tell shape it has changed
	fnPoly.updateSurface();
	
	if(name){
		dgModifier.renameNode( newTransform, name);
	}

	dgModifier.doIt();

	// Put the polygon into a shading group
	MString cmd ("sets -e -fe initialShadingGroup ");
	cmd += fnPoly.name();
	dgModifier.commandToExecute( cmd);

	MFnDagNode fnDagNode( newTransform, &stat );
	if ( MS::kSuccess == stat )	{
		cmd = "select ";
		cmd += fnDagNode.name();
		dgModifier.commandToExecute( cmd);
		cmd = "defaultNavigation -source " + colorTable[colorIndex] + " -destination |" +
					fnDagNode.name() + "|" + fnPoly.name() + ".instObjGroups[0] -connectToExisting;";
		dgModifier.commandToExecute( cmd);
	}

	dgModifier.doIt();


	return MS::kSuccess;
}

MStatus CreateColorTable(std::vector<MString>& colorTable)
{
	MDGModifier			 dgModifier;
	MString name;
	MString cmd;


	//*****************************************************************************
	// White color
	//*****************************************************************************
	name = "WhiteShade";
	colorTable.push_back(name);
	// Result: phong1 //
	cmd = "shadingNode -asShader phong -n " + name + ";";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: phong1SG //
	cmd = "sets -renderable true -noSurfaceShader true -empty -name " + name + "SG;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: Connected phong1.outColor to phong1SG.surfaceShader 
	cmd = "connectAttr -f " + name + ".outColor " + name + "SG.surfaceShader;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	cmd = "setAttr \""+ name + ".color\" -type double3 1 1 1 ;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	//*****************************************************************************
	// Red color
	//*****************************************************************************
	name = "RedShade";
	colorTable.push_back(name);
	// Result: phong1 //
	cmd = "shadingNode -asShader phong -n " + name + ";";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: phong1SG //
	cmd = "sets -renderable true -noSurfaceShader true -empty -name " + name + "SG;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: Connected phong1.outColor to phong1SG.surfaceShader 
	cmd = "connectAttr -f " + name + ".outColor " + name + "SG.surfaceShader;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	cmd = "setAttr \""+ name + ".color\" -type double3 1 0 0 ;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	//*****************************************************************************
	// Green color
	//*****************************************************************************
	name = "GreenShade";
	colorTable.push_back(name);

	// Result: phong1 //
	cmd = "shadingNode -asShader phong -n " + name + ";";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: phong1SG //
	cmd = "sets -renderable true -noSurfaceShader true -empty -name " + name + "SG;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: Connected phong1.outColor to phong1SG.surfaceShader 
	cmd = "connectAttr -f " + name + ".outColor " + name + "SG.surfaceShader;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	cmd = "setAttr \""+ name + ".color\" -type double3 0 1 0 ;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();
	
	
	//*****************************************************************************
	// Blue color
	//*****************************************************************************
	name = "BludeShade";
	colorTable.push_back(name);

	// Result: phong1 //
	cmd = "shadingNode -asShader phong -n " + name + ";";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: phong1SG //
	cmd = "sets -renderable true -noSurfaceShader true -empty -name " + name + "SG;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: Connected phong1.outColor to phong1SG.surfaceShader 
	cmd = "connectAttr -f " + name + ".outColor " + name + "SG.surfaceShader;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	cmd = "setAttr \""+ name + ".color\" -type double3 0 0 1 ;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	//*****************************************************************************
	// Yellow color
	//*****************************************************************************
	name = "YellowShade";
	colorTable.push_back(name);

	// Result: phong1 //
	cmd = "shadingNode -asShader phong -n " + name + ";";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: phong1SG //
	cmd = "sets -renderable true -noSurfaceShader true -empty -name " + name + "SG;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: Connected phong1.outColor to phong1SG.surfaceShader 
	cmd = "connectAttr -f " + name + ".outColor " + name + "SG.surfaceShader;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	cmd = "setAttr \""+ name + ".color\" -type double3 1 1 0 ;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();


	//*****************************************************************************
	// Cyan color
	//*****************************************************************************
	name = "CyanShade";
	colorTable.push_back(name);

	// Result: phong1 //
	cmd = "shadingNode -asShader phong -n " + name + ";";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: phong1SG //
	cmd = "sets -renderable true -noSurfaceShader true -empty -name " + name + "SG;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: Connected phong1.outColor to phong1SG.surfaceShader 
	cmd = "connectAttr -f " + name + ".outColor " + name + "SG.surfaceShader;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	cmd = "setAttr \""+ name + ".color\" -type double3 1 0 1 ;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	//*****************************************************************************
	// Purple color
	//*****************************************************************************
	name = "PurpleShade";
	colorTable.push_back(name);

	// Result: phong1 //
	cmd = "shadingNode -asShader phong -n " + name + ";";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: phong1SG //
	cmd = "sets -renderable true -noSurfaceShader true -empty -name " + name + "SG;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: Connected phong1.outColor to phong1SG.surfaceShader 
	cmd = "connectAttr -f " + name + ".outColor " + name + "SG.surfaceShader;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	cmd = "setAttr \""+ name + ".color\" -type double3 0 1 1 ;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();
	//*****************************************************************************
	// Brown color
	//*****************************************************************************
	name = "BrownShade";
	colorTable.push_back(name);

	// Result: phong1 //
	cmd = "shadingNode -asShader phong -n " + name + ";";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: phong1SG //
	cmd = "sets -renderable true -noSurfaceShader true -empty -name " + name + "SG;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: Connected phong1.outColor to phong1SG.surfaceShader 
	cmd = "connectAttr -f " + name + ".outColor " + name + "SG.surfaceShader;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	cmd = "setAttr \""+ name + ".color\" -type double3 0.5 0.25 0.25 ;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();
	//*****************************************************************************
	// Deep Brown color
	//*****************************************************************************
	name = "DeepBrownShade";
	colorTable.push_back(name);

	// Result: phong1 //
	cmd = "shadingNode -asShader phong -n " + name + ";";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: phong1SG //
	cmd = "sets -renderable true -noSurfaceShader true -empty -name " + name + "SG;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: Connected phong1.outColor to phong1SG.surfaceShader 
	cmd = "connectAttr -f " + name + ".outColor " + name + "SG.surfaceShader;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	cmd = "setAttr \""+ name + ".color\" -type double3 0.5 0 0 ;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	//*****************************************************************************
	// Deep Green color
	//*****************************************************************************
	name = "DeepGreenShade";
	colorTable.push_back(name);

	// Result: phong1 //
	cmd = "shadingNode -asShader phong -n " + name + ";";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: phong1SG //
	cmd = "sets -renderable true -noSurfaceShader true -empty -name " + name + "SG;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: Connected phong1.outColor to phong1SG.surfaceShader 
	cmd = "connectAttr -f " + name + ".outColor " + name + "SG.surfaceShader;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	cmd = "setAttr \""+ name + ".color\" -type double3 0 0.5 0 ;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();
	//*****************************************************************************
	// Black Green color
	//*****************************************************************************
	name = "BlackGreenShade";
	colorTable.push_back(name);

	// Result: phong1 //
	cmd = "shadingNode -asShader phong -n " + name + ";";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: phong1SG //
	cmd = "sets -renderable true -noSurfaceShader true -empty -name " + name + "SG;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: Connected phong1.outColor to phong1SG.surfaceShader 
	cmd = "connectAttr -f " + name + ".outColor " + name + "SG.surfaceShader;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	cmd = "setAttr \""+ name + ".color\" -type double3 0 0.25 0 ;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();
	//*****************************************************************************
	// Black color
	//*****************************************************************************
	name = "BlackShade";
	colorTable.push_back(name);
	// Result: phong1 //
	cmd = "shadingNode -asShader phong -n " + name + ";";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: phong1SG //
	cmd = "sets -renderable true -noSurfaceShader true -empty -name " + name + "SG;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	// Result: Connected phong1.outColor to phong1SG.surfaceShader 
	cmd = "connectAttr -f " + name + ".outColor " + name + "SG.surfaceShader;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();

	cmd = "setAttr \""+ name + ".color\" -type double3 0 0 0 ;";
	dgModifier.commandToExecute( cmd);
	dgModifier.doIt();


	return MS::kSuccess;
}

MStatus SurfaceDraw(char *name, std::vector<vect3<float> > &vertex, unsigned short int colorIndex)
{
	
	// Misc. primitive data
	int num_verts;								// Number of vertices of polygon
	int num_edges;								// Number of edges on polygon
	int num_face_connects;				// Number of elements in face connect array
	int edgePerFaces = 3;
	
	MStatus stat;
	MFnMesh					 fnPoly;			// create a poly function set 
	MFloatPointArray iarr;				// To put the vertices point
	MFloatPointArray pa;
	MIntArray				 faceCounts;
	MIntArray				 faceConnects;

	MObject					 newTransform;
	MDGModifier			 dgModifier;


	vect3<float>     center(0, 0, 0);

	int i;

	for(i = 0; i < vertex.size(); i++)
		center += vertex[i];
	
	center /= (float)vertex.size();
	iarr.clear();
	if(vertex.size() < 3)
		return MS::kFailure;

	iarr.append(center[0], center[1], center[2]);

  // First, create the points:
	for(i = 0; i < vertex.size(); i++){
		MFloatPoint pnt( vertex[i][0], vertex[i][1], vertex[i][2]);
		iarr.append( pnt);
	}

	
	num_verts				 = vertex.size() + 1;
	num_edges        = vertex.size() * 2;

	// Construct the point array
	pa.clear();
	for (i = 0; i < num_verts; i++){
		pa.append(iarr[i]);
	}


	for ( i = 0; i < vertex.size(); i++ ){
		faceCounts.append(edgePerFaces);
		int index0, index1;
		index0 = i + 1; 
	
		index1 = i + 2;
		if(index1 > vertex.size())
			index1 = 1;
		faceConnects.append(0);
		faceConnects.append(index0);
		faceConnects.append(index1);
	}


	// Call the poly creation method to create the polygon
	newTransform = fnPoly.create(num_verts, vertex.size(), pa, faceCounts, faceConnects, MObject::kNullObj, &stat );

	checkErr(stat, "Could not create MFnMesh");

	// Primitive is created so tell shape it has changed
	fnPoly.updateSurface();
	
	if(name){
		dgModifier.renameNode( newTransform, name);
	}

	dgModifier.doIt();

	// Put the polygon into a shading group
	MString cmd ("sets -e -fe initialShadingGroup ");
	cmd += fnPoly.name();
	dgModifier.commandToExecute( cmd);

	MFnDagNode fnDagNode( newTransform, &stat );
	if ( MS::kSuccess == stat )	{
		cmd = "select ";
		cmd += fnDagNode.name();
		dgModifier.commandToExecute( cmd);
		cmd = "defaultNavigation -source " + colorTable[colorIndex] + " -destination |" +
					fnDagNode.name() + "|" + fnPoly.name() + ".instObjGroups[0] -connectToExisting;";
		dgModifier.commandToExecute( cmd);
	}

	dgModifier.doIt();

	return MS::kSuccess;
}
